package com.marvinelsen.willow

import com.marvinelsen.willow.domain.objects.DictionaryEntry
import com.marvinelsen.willow.domain.objects.Sentence
import com.marvinelsen.willow.ui.undo.UndoManager
import javafx.beans.property.BooleanProperty
import javafx.beans.property.ObjectProperty
import javafx.beans.property.ReadOnlyBooleanProperty
import javafx.beans.property.SimpleBooleanProperty
import javafx.beans.property.SimpleObjectProperty
import javafx.collections.FXCollections
import javafx.collections.ObservableList

@Suppress("TooManyFunctions")
class Model(
    val undoManager: UndoManager,
) {
    private val _selectedEntry: ObjectProperty<DictionaryEntry?> = SimpleObjectProperty<DictionaryEntry>()

    private val _isSearching: BooleanProperty = SimpleBooleanProperty(false)
    private val _isFindingWordsBeginning: BooleanProperty = SimpleBooleanProperty(false)
    private val _isFindingWordsContaining: BooleanProperty = SimpleBooleanProperty(false)
    private val _isFindingCharacters: BooleanProperty = SimpleBooleanProperty(false)
    private val _isFindingSentences: BooleanProperty = SimpleBooleanProperty(false)

    private val _finishedFindingWordsBeginning: BooleanProperty = SimpleBooleanProperty(false)
    private val _finishedFindingWordsContaining: BooleanProperty = SimpleBooleanProperty(false)
    private val _finishedFindingCharacters: BooleanProperty = SimpleBooleanProperty(false)
    private val _finishedFindingSentences: BooleanProperty = SimpleBooleanProperty(false)

    val searchResults: ObservableList<DictionaryEntry> = FXCollections.observableArrayList()
    val wordsBeginning: ObservableList<DictionaryEntry> = FXCollections.observableArrayList()
    val wordsContaining: ObservableList<DictionaryEntry> = FXCollections.observableArrayList()
    val characters: ObservableList<DictionaryEntry> = FXCollections.observableArrayList()
    val sentences: ObservableList<Sentence> = FXCollections.observableArrayList()

    val canUndo: ReadOnlyBooleanProperty = undoManager.canUndoProperty
    val canRedo: ReadOnlyBooleanProperty = undoManager.canRedoProperty

    var selectedEntry: DictionaryEntry?
        get() = _selectedEntry.value
        set(value) {
            _selectedEntry.value = value
        }

    var isSearching: Boolean
        get() = _isSearching.value
        set(value) {
            _isSearching.value = value
        }

    var isFindingWordsBeginning: Boolean
        get() = _isFindingWordsBeginning.value
        set(value) {
            _isFindingWordsBeginning.value = value
        }

    var isFindingWordsContaining: Boolean
        get() = _isFindingWordsContaining.value
        set(value) {
            _isFindingWordsContaining.value = value
        }

    var isFindingCharacters: Boolean
        get() = _isFindingCharacters.value
        set(value) {
            _isFindingCharacters.value = value
        }

    var isFindingSentences: Boolean
        get() = _isFindingSentences.value
        set(value) {
            _isFindingSentences.value = value
        }

    var finishedFindingWordsBeginning: Boolean
        get() = _finishedFindingWordsBeginning.value
        set(value) {
            _finishedFindingWordsBeginning.value = value
        }

    var finishedFindingWordsContaining: Boolean
        get() = _finishedFindingWordsContaining.value
        set(value) {
            _finishedFindingWordsContaining.value = value
        }

    var finishedFindingCharacters: Boolean
        get() = _finishedFindingCharacters.value
        set(value) {
            _finishedFindingCharacters.value = value
        }

    var finishedFindingSentences: Boolean
        get() = _finishedFindingSentences.value
        set(value) {
            _finishedFindingSentences.value = value
        }

    fun selectedEntryProperty(): ObjectProperty<DictionaryEntry?> = _selectedEntry
    fun isSearchingProperty(): BooleanProperty = _isSearching
    fun isFindingWordsBeginningProperty(): BooleanProperty = _isFindingWordsBeginning
    fun isFindingWordsContainingProperty(): BooleanProperty = _isFindingWordsContaining
    fun isFindingCharactersProperty(): BooleanProperty = _isFindingCharacters
    fun isFindingSentencesProperty(): BooleanProperty = _isFindingSentences
    fun finishedFindingWordsBeginning(): BooleanProperty = _finishedFindingWordsBeginning
    fun finishedFindingWordsContaining(): BooleanProperty = _finishedFindingWordsContaining
    fun finishedFindingCharacters(): BooleanProperty = _finishedFindingCharacters
    fun finishedFindingSentences(): BooleanProperty = _finishedFindingSentences
}
