package com.marvinelsen.willow.ui.components

import com.marvinelsen.willow.config.Config
import com.marvinelsen.willow.config.Script
import com.marvinelsen.willow.domain.objects.DictionaryEntry
import javafx.beans.property.SimpleObjectProperty
import javafx.scene.paint.Color
import javafx.scene.text.Text
import javafx.scene.text.TextFlow

class TextFlowWithToneColors(private val config: Config) : TextFlow() {
    init {
        config.script.addListener { _, _, newValue ->
            updateTextFlow(newValue)
        }
    }

    val entry = SimpleObjectProperty<DictionaryEntry?>().apply {
        addListener { _, _, _ ->
            updateTextFlow(config.script.value)
        }
    }

    private val invalidCharacterRegex = """[a-zA-Z0-9，．·\[\]{}()+]""".toRegex()

    @Suppress("MagicNumber")
    private fun updateTextFlow(script: Script) {
        children.clear()

        val headword = when (script) {
            Script.SIMPLIFIED -> entry.value?.simplified
            Script.TRADITIONAL -> entry.value?.traditional
        } ?: return

        var index = 0
        for (char in headword) {
            val text = Text(char.toString())

            val color = if (invalidCharacterRegex.matches(char.toString())) {
                Color.BLACK
            } else {
                when (entry.value?.tones?.getOrNull(index++)) {
                    1 -> Color.RED
                    2 -> Color.GREEN
                    3 -> Color.BLUE
                    4 -> Color.PURPLE
                    5 -> Color.DARKGRAY
                    else -> Color.BLACK
                }
            }
            text.fill = color
            children.add(text)
        }
    }
}
