package com.marvinelsen.willow.ui.controllers

import com.marvinelsen.willow.Interactor
import com.marvinelsen.willow.Model
import com.marvinelsen.willow.config.Config
import com.marvinelsen.willow.domain.objects.DictionaryEntry
import com.marvinelsen.willow.ui.cells.DictionaryEntryCellFactory
import javafx.application.HostServices
import javafx.beans.binding.Bindings
import javafx.fxml.FXML
import javafx.scene.control.Label
import javafx.scene.control.ListView
import javafx.scene.control.ProgressIndicator
import java.util.ResourceBundle

class SearchResultsController(
    private val model: Model,
    private val interactor: Interactor,
    private val config: Config,
    private val hostServices: HostServices,
) {

    @FXML
    private lateinit var resources: ResourceBundle

    @FXML
    private lateinit var progressIndicatorEntries: ProgressIndicator

    @FXML
    @Suppress("UnusedPrivateProperty")
    private lateinit var labelNoEntriesFound: Label

    @FXML
    private lateinit var listViewSearchResults: ListView<DictionaryEntry>

    @FXML
    @Suppress("UnusedPrivateMember")
    private fun initialize() {
        listViewSearchResults.cellFactory = DictionaryEntryCellFactory(resources, config, hostServices)
        listViewSearchResults.items = model.searchResults
        listViewSearchResults
            .disableProperty()
            .bind(Bindings.or(model.isSearchingProperty(), Bindings.isEmpty(model.searchResults)))

        progressIndicatorEntries.visibleProperty().bind(model.isSearchingProperty())
        labelNoEntriesFound
            .visibleProperty()
            .bind(Bindings.and(Bindings.isEmpty(model.searchResults), Bindings.not(model.isSearchingProperty())))

        listViewSearchResults.selectionModel.selectedItemProperty().addListener { _, _, newValue: DictionaryEntry? ->
            if (newValue == null) {
                return@addListener
            }
            interactor.normalSelect(newValue)
        }
    }
}
